#ifndef SIMULATEUR_H
#define SIMULATEUR_H

#include <map>
#include <vector>

constexpr size_t kNb_joueuses{10};
constexpr size_t kNb_total_cartes{104};
constexpr size_t kNb_initial_cartes_en_main{10};
constexpr size_t kNb_rangees{4};
constexpr size_t kNb_max_cartes_dans_rangee{5};
constexpr size_t kNb_simulations{1};

enum class strategie_t {
  kCarte_la_plus_petite,
  kCarte_au_milieu_main,
  kCarte_random,
  kCarte_avant_la_plus_grande,
  kCarte_la_plus_grande
};

using cartes_t = std::vector<int>;
using rangees_t = std::vector<cartes_t>;
using statistiques_t =
  std::map<strategie_t, std::pair<int, int>>; // Dans la paire, first contient la somme
                                      // des rangs, second la somme des vaches.

struct Joueuse {
  strategie_t strategie{0};
  cartes_t main;
  int carte_pour_ce_tour{0};
  cartes_t cartes_encaissees;
  int nb_vaches_encaissees{0}; // Selon https://www.regledujeu.fr/6-qui-prend/,
                               // on aurait pu parler de Tetes de boeufs.
};

/**
 * @brief Met dans @a j.carte_pour_ce_tour la carte que jouera la joueuse a ce tour, en fonction de @a j.strategie. Supprime cette carte de @a j.main.
 * @param[in/out] j Joueuse sur laquelle travailler.
 * @attention Pre-condition : @a j.main est supposee triee par ordre croissant. 
*/
void choisir_carte(Joueuse j);// TODO 4 bis : Decidez si j est passé en copie, en référence ou en référence constante.

/**
 * @brief Affiche les differentes elements de @p cartes.
 * @param[in] cartes Cartes dont on veut afficher les valeurs.
 */
void dump_cartes(const cartes_t &cartes);

/**
 * @brief Retourne le nombre de vaches encaissees sur une carte.
 * @param[in] carte Valeur de la carte.
 * @details Fonction implantant les regles definies en secion "3) Précisions sur 
 * les Têtes de Boeufs" de https://www.regledujeu.fr/6-qui-prend/.
 * @return Nombre de vaches encaissees.
 */
int get_nb_vaches(int carte);

/**
* @brief Retourne le nombre de vaches encaissees dans le paquet de cartes @cartes.
* @return Nombre total de vaches encaissees.
*/
int get_nb_vaches(const cartes_t& cartes);

/**
 * @brief Retourne le nom correspondant a @p strategie.
 * @param[in] strategie Strategie dont on cherche le nom.
 * @return Nom correspondant a @p strategie.
 */
std::string_view get_nom_strategie(strategie_t strategie);

/**
 * @brief Determine le rang de @p rangees ou @p j joue.
 * @param[in] j Joueuse pour laquelle on cherche le rang ou jouer.
 * @param[in] rangees Rangees parmi lesquelles @p j peut jouer.
 * @return Rang retenu.
 */
size_t get_rang_ou_jouer(const Joueuse& j, const rangees_t& rangees);

/**
 * @brief Joue @a j.carte_pour_ce_tour dans la bonne rangee et modifie @a j.cartes_encaissees et
 * @param[in/out] j Joueuse qui joue sa carte. Cette procedure peut modifer @a j.cartes_encaissees et j.nb_vachesçencaissees.
 * @param[in] rangees Rangees parmi lesquelles @p j va jouer. La rangee jouee voit son contenu evoluer en fonction des règles du jeu.
 */
void jouer_carte(Joueuse j, rangees_t rangees);  // TODO 6 bis : Decidez si j et rangees sont passés en copie, en référence ou en référence constante.

/**
 * @brief Simule une partie et met a jour @p stat.
 * @param[in/out] stat Statistiques a mettre a jour avec les resultats d'une simulation.
 */
void simuler_partie(statistiques_t& stat);

#endif // SIMULATEUR_H
